/*
 * simio.c --
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "common.h"
#include "pipe.h"
#include "arith.h"


typedef struct {
  FILE *fp;
  int mode;
} sim_file_info;


sim_file_info sfile[256];

char *mode_string[] = { "r", "w", "rb", "wb", "w+b" };
int mode_code[] = { 0x1, 0x2, 0x5, 0x6, 0xf };

char *trip_warning[] = {
  "TRIP",
  "integer divide check",
  "integer overflow",
  "float-to-fix overflow",
  "invalid floating point operation",
  "floating point overflow",
  "floating point underflow",
  "floating point division by zero",
  "floating point inexact"
};


void simioInit(void) {
  sfile[0].fp = stdin;
  sfile[0].mode = 1;
  sfile[1].fp = stdout;
  sfile[1].mode = 2;
  sfile[2].fp = stderr;
  sfile[2].mode = 2;
}


Octa simioFopen(unsigned char handle, Octa name, Octa mode) {
  char name_buf[FILENAME_MAX];

  if (mode.h || mode.l > 4) {
    goto abort;
  }
  if (mmgetchars(name_buf, FILENAME_MAX, name, 0) == FILENAME_MAX) {
    goto abort;
  }
  if (sfile[handle].mode != 0 && handle > 2) {
    fclose(sfile[handle].fp);
  }
  sfile[handle].fp = fopen(name_buf, mode_string[mode.l]);
  if (!sfile[handle].fp) {
    goto abort;
  }
  sfile[handle].mode = mode_code[mode.l];
  return zero;
abort: ;
  sfile[handle].mode= 0;
  return negOne;
}


void simioFakeStdin(FILE *f) {
  sfile[0].fp = f;
}


Octa simioFclose(unsigned char handle) {
  if (sfile[handle].mode ==0 ) {
    return negOne;
  }
  if (handle > 2 && fclose(sfile[handle].fp) != 0) {
    return negOne;
  }
  sfile[handle].mode = 0;
  return zero;
}


Octa simioFread(unsigned char handle, Octa buffer, Octa size) {
  unsigned char *buf;
  int n;
  Octa o;

  o = negOne;
  if (!(sfile[handle].mode&0x1)) goto done;
  if (sfile[handle].mode&0x8) sfile[handle].mode &= ~0x2;
  if (size.h) goto done;
  buf = (unsigned char *) calloc(size.l, sizeof(char));
  if (!buf) goto done;
  if (sfile[handle].fp == stdin) {
    unsigned char *p;
    for (p = buf, n = size.l; p < buf + n; p++) *p= stdin_chr();
  } else {
    clearerr(sfile[handle].fp);
    n = fread(buf, 1, size.l, sfile[handle].fp);
    if (ferror(sfile[handle].fp)) {
      free(buf);
      goto done;
    }
  }
  mmputchars(buf,n,buffer);
  free(buf);
  o.h = 0, o.l = n;
done: ;
  return ominus(o, size);
}


Octa simioFgets(handle,buffer,size)
unsigned char handle;
Octa buffer,size;
{
char buf[256];
register int n,s;
register char*p;
Octa o;
int eof= 0;
if(!(sfile[handle].mode&0x1))return negOne;
if(!size.l&&!size.h)return negOne;
if(sfile[handle].mode&0x8)sfile[handle].mode&= ~0x2;
size= incr(size,-1);
o= zero;
while(1){
s= 255;
if(size.l<s&&!size.h)s= size.l;
if(sfile[handle].fp==stdin)
for(p= buf,n= 0;n<s;){
*p= stdin_chr();
n++;
if(*p++=='\n')break;
}
else{
if(!fgets(buf,s+1,sfile[handle].fp))return negOne;
eof= feof(sfile[handle].fp);
for(p= buf,n= 0;n<s;){
if(!*p&&eof)break;
n++;
if(*p++=='\n')break;
}
}
*p= '\0';
mmputchars(buf,n+1,buffer);
o= incr(o,n);
size= incr(size,-n);
if((n&&buf[n-1]=='\n')||(!size.l&&!size.h)||eof)return o;
buffer= incr(buffer,n);
}
}


Octa simioFgetws(handle,buffer,size)
unsigned char handle;
Octa buffer,size;
{
char buf[256];
register int n,s;
register char*p;
Octa o;
int eof;
if(!(sfile[handle].mode&0x1))return negOne;
if(!size.l&&!size.h)return negOne;
if(sfile[handle].mode&0x8)sfile[handle].mode&= ~0x2;
buffer.l&= -2;
size= incr(size,-1);
o= zero;
while(1){
s= 127;
if(size.l<s&&!size.h)s= size.l;
if(sfile[handle].fp==stdin)
for(p= buf,n= 0;n<s;){
*p++= stdin_chr();*p++= stdin_chr();
n++;
if(*(p-1)=='\n'&&*(p-2)==0)break;
}
else for(p= buf,n= 0;n<s;){
if(fread(p,1,2,sfile[handle].fp)!=2){
eof= feof(sfile[handle].fp);
if(!eof)return negOne;
break;
}
n++,p+= 2;
if(*(p-1)=='\n'&&*(p-2)==0)break;
}
*p= *(p+1)= '\0';
mmputchars(buf,2*n+2,buffer);
o= incr(o,n);
size= incr(size,-n);
if((n&&buf[2*n-1]=='\n'&&buf[2*n-2]==0)||(!size.l&&!size.h)||eof)
return o;
buffer= incr(buffer,2*n);
}
}


Octa simioFwrite(handle,buffer,size)
unsigned char handle;
Octa buffer,size;
{
char buf[256];
register int n;
if(!(sfile[handle].mode&0x2))return ominus(zero,size);
if(sfile[handle].mode&0x8)sfile[handle].mode&= ~0x1;
while(1){
if(size.h||size.l>=256)n= mmgetchars(buf,256,buffer,-1);
else n= mmgetchars(buf,size.l,buffer,-1);
size= incr(size,-n);
if(fwrite(buf,1,n,sfile[handle].fp)!=n)return ominus(zero,size);
fflush(sfile[handle].fp);
if(!size.l&&!size.h)return zero;
buffer= incr(buffer,n);
}
}


Octa simioFputs(handle,string)
unsigned char handle;
Octa string;
{
char buf[256];
register int n;
Octa o;
o= zero;
if(!(sfile[handle].mode&0x2))return negOne;
if(sfile[handle].mode&0x8)sfile[handle].mode&= ~0x1;
while(1){
n= mmgetchars(buf,256,string,0);
if(fwrite(buf,1,n,sfile[handle].fp)!=n)return negOne;
o= incr(o,n);
if(n<256){
fflush(sfile[handle].fp);
return o;
}
string= incr(string,n);
}
}


Octa simioFputws(handle,string)
unsigned char handle;
Octa string;
{
char buf[256];
register int n;
Octa o;
o= zero;
if(!(sfile[handle].mode&0x2))return negOne;
if(sfile[handle].mode&0x8)sfile[handle].mode&= ~0x1;
while(1){
n= mmgetchars(buf,256,string,1);
if(fwrite(buf,1,n,sfile[handle].fp)!=n)return negOne;
o= incr(o,n>>1);
if(n<256){
fflush(sfile[handle].fp);
return o;
}
string= incr(string,n);
}
}


Octa simioFseek(unsigned char handle, Octa offset) {
  if (!(sfile[handle].mode & 0x4)) {
    return negOne;
  }
  if (sfile[handle].mode & 0x8) {
    sfile[handle].mode = 0xf;
  }
  if (offset.h & SIGN_BIT) {
    if (offset.h != 0xffffffff || !(offset.l & SIGN_BIT)) {
      return negOne;
    }
    if (fseek(sfile[handle].fp, (int) offset.l + 1, SEEK_END) != 0) {
      return negOne;
    }
  } else {
    if (offset.h || (offset.l & SIGN_BIT)) {
      return negOne;
    }
    if (fseek(sfile[handle].fp, (int) offset.l, SEEK_SET) !=0) {
      return negOne;
    }
  }
  return zero;
}


Octa simioFtell(unsigned char handle) {
  long x;
  Octa o;

  if (!(sfile[handle].mode & 0x4)) {
    return negOne;
  }
  x = ftell(sfile[handle].fp);
  if (x < 0) {
    return negOne;
  }
  o.h = 0;
  o.l = x;
  return o;
}


void printTripWarning(int n, Octa loc) {
  if (sfile[2].mode & 0x2) {
    fprintf(sfile[2].fp, "Warning: %s at location %08x%08x\n",
            trip_warning[n], loc.h, loc.l);
  }
}
