/*
 * common.h -- common definitions
 */


#ifndef _COMMON_H_
#define _COMMON_H_


#define VERSION		1		/* version of the MMIX architecture */
#define SUBVERSION	0		/* secondary byte of version number */
#define SUBSUBVERSION	2		/* further qualification of version */

#define K		1024		/* Kilo */
#define M		(K * K)		/* Mega */

#define GR_LOG2		8		/* log2 of number of global regs */
#define GR_NUM		(1 << GR_LOG2)	/* number of global registers */
#define GR_MASK		(GR_NUM - 1)	/* mask for global registers */

#define LR_LOG2		9		/* log2 of number of local regs */
#define LR_NUM		(1 << LR_LOG2)	/* number of local registers */
#define LR_MASK		(LR_NUM - 1)	/* mask for local registers */

#define MEMSIZE_MIN	(1 * M)		/* minimum memory size */
#define MEMSIZE_MAX	(16 * M)	/* maximum memory size */
#define MEMSIZE_DEF	(4 * M)		/* default memory size */
#define MEM_BASE_H	0x00000000	/* phys base addr of mem, hi part */
#define MEM_BASE_L	0x00000000	/* ditto, lo part, must be 0 */

#define ROM_SIZE	(64 * K)	/* ROM size */
#define ROM_BASE_H	0x00008000	/* phys base addr of ROM, hi part */
#define ROM_BASE_L	0x00000000	/* ditto, lo part, must be 0 */

#define DEV_TYPE_MASK	0xFFFF0000	/* device type bits in hi addr */
#define DEV_NUM_MASK	0x0000FFFF	/* device number bits in hi addr */

#define TERM_BASE_H	0x00010000	/* terminal base addr, hi part */
#define TERM_BASE_L	0x00000000	/* ditto, lo part, must be 0 */
#define DISK_BASE_H	0x00020000	/* disk base addr, hi part */
#define DISK_BASE_L	0x00000000	/* ditto, lo part, must be 0 */
#define GRAPH_BASE_H	0x00030000	/* graphics base addr, hi part */
#define GRAPH_BASE_L	0x00000000	/* ditto, lo part, must be 0 */

#define NTERMS_MIN	0		/* minimum number of terminals */
#define NTERMS_MAX	4		/* maximum number of terminals */
#define NTERMS_DEF	1		/* default number of terminals */

#define INSTRS_PER_MSEC	1500		/* average execution speed */

#define SIGN(n)		(((Tetra) 1) << ((n) - 1))	/* n = 1..32 */
#define MASK(n)		((((Tetra) 1) << (n)) - 1)	/* n = 0..31 */
#define ZEXT(x,n)	(((Tetra) (x)) & MASK(n))
#define SEXT(x,n)	((((Tetra) (x)) & SIGN(n)) ? \
			 (((Tetra) (x)) | ~MASK(n)) : \
			 (((Tetra) (x)) & MASK(n)))


typedef int Bool;			/* truth values */

#define FALSE		0
#define TRUE		1


typedef unsigned char U8;
typedef signed char S8;
typedef unsigned short U16;
typedef signed short S16;
typedef unsigned int U32;
typedef signed int S32;


typedef U8 Byte;			/* 8-bit quantities */
typedef U16 Wyde;			/* 16-bit quantities */
typedef U32 Tetra;			/* 32-bit quantities */
typedef struct {
  Tetra h;
  Tetra l;
} Octa;					/* 64-bit quantities */


#endif /* _COMMON_H_ */
