/*
 * except.c -- exception handling
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <setjmp.h>

#include "error.h"
#include "except.h"


#define MAX_ENV_NEST_DEPTH	10


static jmp_buf *environments[MAX_ENV_NEST_DEPTH];
static int currentEnvironment = -1;


static char *negExcStrings[] = {
  /* -0 */  "No exception",
  /* -1 */  "A trap instruction exception",
  /* -2 */  "An emulated instruction exception",
  /* -3 */  "A virtual address translation exception"
};


static char *posExcStrings[] = {
  /*  0 */  "No exception",
  /* machine exceptions */
  /*  1 */  "A power failure exception (1)",
  /*  2 */  "A parity error exception (2)",
  /*  3 */  "A nonexistent memory exception (3)",
  /*  4 */  "A write-protect exception (4)",
  /*  5 */  "A reboot signal exception (5)",
  /*  6 */  "A dynamic trap exception (6)",
  /*  7 */  "A dynamic trap exception (7)",
  /*  8 */  "An interval timeout exception (8)",
  /* high-priority I/O exceptions */
  /*  9 */  "A dynamic trap exception (9)",
  /* 10 */  "A dynamic trap exception (10)",
  /* 11 */  "A dynamic trap exception (11)",
  /* 12 */  "A dynamic trap exception (12)",
  /* 13 */  "A dynamic trap exception (13)",
  /* 14 */  "A dynamic trap exception (14)",
  /* 15 */  "A dynamic trap exception (15)",
  /* 16 */  "A dynamic trap exception (16)",
  /* 17 */  "A dynamic trap exception (17)",
  /* 18 */  "A dynamic trap exception (18)",
  /* 19 */  "A dynamic trap exception (19)",
  /* 20 */  "A dynamic trap exception (20)",
  /* 21 */  "A dynamic trap exception (21)",
  /* 22 */  "A dynamic trap exception (22)",
  /* 23 */  "A dynamic trap exception (23)",
  /* 24 */  "A dynamic trap exception (24)",
  /* 25 */  "A dynamic trap exception (25)",
  /* 26 */  "A dynamic trap exception (26)",
  /* 27 */  "A dynamic trap exception (27)",
  /* 28 */  "A dynamic trap exception (28)",
  /* 29 */  "A dynamic trap exception (29)",
  /* 30 */  "A dynamic trap exception (30)",
  /* 31 */  "A dynamic trap exception (31)",
  /* 32 */  "A dynamic trap exception (32)",
  /* program exceptions */
  /* 33 */  "A program p-bit exception (33)",
  /* 34 */  "A program s-bit exception (34)",
  /* 35 */  "A program b-bit exception (35)",
  /* 36 */  "A program k-bit exception (36)",
  /* 37 */  "A program n-bit exception (37)",
  /* 38 */  "A program x-bit exception (38)",
  /* 39 */  "A program w-bit exception (39)",
  /* 40 */  "A program r-bit exception (40)",
  /* low-priority I/O exceptions */
  /* 41 */  "A dynamic trap exception (41)",
  /* 42 */  "A dynamic trap exception (42)",
  /* 43 */  "A dynamic trap exception (43)",
  /* 44 */  "A dynamic trap exception (44)",
  /* 45 */  "A dynamic trap exception (45)",
  /* 46 */  "A dynamic trap exception (46)",
  /* 47 */  "A dynamic trap exception (47)",
  /* 48 */  "A dynamic trap exception (48)",
  /* 49 */  "A dynamic trap exception (49)",
  /* 50 */  "A dynamic trap exception (50)",
  /* 51 */  "A dynamic trap exception (51)",
  /* 52 */  "A dynamic trap exception (52)",
  /* 53 */  "A dynamic trap exception (53)",
  /* 54 */  "A dynamic trap exception (54)",
  /* 55 */  "A dynamic trap exception (55)",
  /* 56 */  "A dynamic trap exception (56)",
  /* 57 */  "A dynamic trap exception (57)",
  /* 58 */  "A dynamic trap exception (58)",
  /* 59 */  "A dynamic trap exception (59)",
  /* 60 */  "A dynamic trap exception (60)",
  /* 61 */  "A dynamic trap exception (61)",
  /* 62 */  "A dynamic trap exception (62)",
  /* 63 */  "A dynamic trap exception (63)",
  /* 64 */  "A dynamic trap exception (64)"
};


char *exceptionToString(int exception) {
  if (exception < 0) {
    exception = -exception;
    if (exception >= sizeof(negExcStrings) / sizeof(negExcStrings[0])) {
      error("negative exception number out of bounds");
    }
    return negExcStrings[exception];
  } else {
    if (exception >= sizeof(posExcStrings) / sizeof(posExcStrings[0])) {
      error("positive exception number out of bounds");
    }
    return posExcStrings[exception];
  }
}


void throwException(int exception) {
  if (currentEnvironment < 0) {
    error("exception %d thrown while no environment active", exception);
  }
  longjmp(*environments[currentEnvironment], exception);
}


void pushEnvironment(jmp_buf *environment) {
  if (currentEnvironment == MAX_ENV_NEST_DEPTH - 1) {
    error("too many environments active");
  }
  currentEnvironment++;
  environments[currentEnvironment] = environment;
}


void popEnvironment(void) {
  if (currentEnvironment < 0) {
    error("cannot pop environment - none active");
  }
  currentEnvironment--;
}
