/*
 * mmu.h -- MMU simulation
 */


#ifndef _MMU_H_
#define _MMU_H_


#define ITC		0		/* instr translation cache */
#define DTC		1		/* data translation cache */

#define ITC_SIZE	32		/* number of entries in ITC */
#define DTC_SIZE	32		/* number of entries in DTC */


typedef struct {
  Bool valid;		/* true iff this entry is valid */
  Octa page;		/* virtual page address */
  int asn;		/* address space number */
  Octa frame;		/* physical frame address */
  int prot;		/* protection code */
} TC_Entry;


Tetra mmuGetInstr(Octa vAddr);
void mmuPutInstr(Octa vAddr, Tetra instr);

Octa mmuReadOcta(Octa vAddr);
Tetra mmuReadTetra(Octa vAddr);
Wyde mmuReadWyde(Octa vAddr);
Byte mmuReadByte(Octa vAddr);
void mmuWriteOcta(Octa vAddr, Octa data);
void mmuWriteTetra(Octa vAddr, Tetra data);
void mmuWriteWyde(Octa vAddr, Wyde data);
void mmuWriteByte(Octa vAddr, Byte data);

void mmuCtrl(int s, int n);
int mmuUpdate(Octa key);

void mmuShowAll(int cacheID);
void mmuRemoveAll(int cacheID);
void mmuShowEntry(int cacheID, Octa key);
void mmuRemoveEntry(int cacheID, Octa key);
void mmuInsertEntry(int cacheID, Octa key, Octa xlt);

void mmuReset(void);
void mmuInit(void);
void mmuExit(void);


#endif /* _MMU_H_ */
