/*
 * alwrap.c -- a wrapper program for calling the assembler
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <unistd.h>


#define CODE_SEGMENT	"% -- CODE SEGMENT --"
#define DATA_SEGMENT	"% -- DATA SEGMENT --"
#define PREFIX_CHANGE	"% -- PREFIX CHANGE --"

#define LINE_SIZE	200

#define CODE		0
#define DATA		1


char codeName[20];
FILE *codeFile = NULL;
char dataName[20];
FILE *dataFile = NULL;


void error(char *fmt, ...) {
  va_list ap;

  if (codeFile != NULL) {
    fclose(codeFile);
    unlink(codeName);
  }
  if (dataFile != NULL) {
    fclose(dataFile);
    unlink(dataName);
  }
  va_start(ap, fmt);
  fprintf(stderr, "Error: ");
  vfprintf(stderr, fmt, ap);
  fprintf(stderr, "\n");
  va_end(ap);
  exit(1);
}


int main(int argc, char *argv[]) {
  int i, n;
  FILE *inputFile;
  char line[LINE_SIZE];
  int segment;
  char cmd[5 * LINE_SIZE];

  if (argc < 4) {
    fprintf(stderr,
            "Usage: %s <assembler> [options] --files <file1> ...\n",
            argv[0]);
    exit(1);
  }
  strcpy(codeName, "./code_XXXXXX");
  mktemp(codeName);
  codeFile = fopen(codeName, "w+");
  if (codeFile == NULL) {
    error("cannot open temporary code file '%s'", codeName);
  }
  strcpy(dataName, "./data_XXXXXX");
  mktemp(dataName);
  dataFile = fopen(dataName, "w+");
  if (dataFile == NULL) {
    error("cannot open temporary data file '%s'", dataName);
  }
  for (i = 2; i < argc - 1; i++) {
    if (strcmp(argv[i], "--files") == 0) {
      break;
    }
  }
  if (i == argc - 1) {
    error("option --input ... is missing");
  }
  n = i++;
  fprintf(dataFile, "8H\tLOC\t9B\t\t%% -- data segment starting here --\n");
  for (; i < argc; i++) {
    inputFile = fopen(argv[i], "r");
    if (inputFile == NULL) {
      error("cannot open input file '%s'", argv[i]);
    }
    segment = DATA;
    while (fgets(line, LINE_SIZE, inputFile) != NULL) {
      if (strstr(line, CODE_SEGMENT) != NULL) {
        segment = CODE;
      } else
      if (strstr(line, DATA_SEGMENT) != NULL) {
        segment = DATA;
      } else
      if (strstr(line, PREFIX_CHANGE) != NULL) {
        fprintf(codeFile, "%s", line);
        fprintf(dataFile, "%s", line);
      } else {
        if (segment == CODE) {
          fprintf(codeFile, "%s", line);
        } else
        if (segment == DATA) {
          fprintf(dataFile, "%s", line);
        } else {
          error("this cannot happen");
        }
      }
    }
    if (segment != DATA) {
      error("file '%s' does not end in data segment", argv[i]);
    }
    fclose(inputFile);
  }
  fseek(codeFile, 0, SEEK_SET);
  fprintf(dataFile, "9H\tLOC\t8B\t\t%% -- code segment starting here --\n");
  while (fgets(line, LINE_SIZE, codeFile) != NULL) {
    fprintf(dataFile, "%s", line);
  }
  fclose(codeFile);
  unlink(codeName);
  fclose(dataFile);
  strcpy(cmd, argv[1]);
  for (i = 2; i < n; i++) {
    strcat(cmd, " ");
    strcat(cmd, argv[i]);
  }
  strcat(cmd, " ");
  strcat(cmd, dataName);
  if (system(cmd) != 0) {
    error("cannot execute assembler");
  }
  unlink(dataName);
  return 0;
}
